#include "cquad.h"

double cquad_dqawce(double f(), double a, double b, double c, double epsabs, double epsrel, double *abserr, int *neval, int *ier, double *alist, double *blist, double *rlist, double *elist, int *iord, int *last)
{
/******************************************************************************
***  Date written:         03/31/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/02/2012  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@unina.it)                ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***									    ***
*** 									    ***
***									    ***
*** This code is provided "as is", without any warranty of any kind,        ***
*** either expressed or implied, including but not limited to, any implied  ***
*** warranty of merchantibility or fitness for any purpose. In no event     ***
*** will any party who distributed the code be liable for damages or for    ***
*** any claim(s) by any other party, including but not limited to, any      ***
*** lost profits, lost monies, lost data or data rendered inaccurate,	    ***
*** losses sustained by third parties, or any other special, incidental or  ***	
*** consequential damages arising out of the use or inability to use the    ***
*** program, even if the possibility of such damages has been advised	    ***	
*** against. The entire risk as to the quality, the performance, and the    ***	
*** fitness of the program for any particular purpose lies with the party   ***
*** using the code.							    ***
***									    ***
*** This code, and any derivative of this code, may not be used in a        ***
*** commercial package without the prior explicit written permission of     ***
*** the authors. Verbatim copies of this code may be made and distributed   ***
*** in any medium, provided that this copyright notice is not removed or    ***
*** altered in any way. No fees may be charged for distribution of the      ***
*** codes, other than a fee to cover the cost of the media and a            ***
*** reasonable handling fee.						    ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    Computes the Cauchy Principal Value of f(x) / (x - c) over a finite  ***
***    interval (a, b) and for user-determined c.                           ***
***    The strategy is globally adaptive, and modified Clenshaw-Curtis      ***
***    integration is used on the subranges which contain the point x = c.  ***
***                                                                         ***
***    Expert user version (provide more information and control than the   ***
***    easier version cquad_dqawc).                                         ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    b        upper limit of integration (b > a)                          ***
***                                                                         ***
***    c        parameter in the weight function (c != a, c != b)           ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> maximum number of subdivisions allowed has    ***
***                           been achieved.                                ***
***                                                                         ***
***               ier = 2 --> the occurrence of roundoff error is detected  ***
***                           which prevents the requested tolerance from   ***
***                           being achieved.                               ***
***                                                                         ***
***               ier = 3 --> extremely bad integrand behaviour occurs at   ***
***                           some interior points of the integration       ***
***                           interval.                                     ***
***                                                                         ***
***               ier = 6 --> the input is invalid because (epsabs <= 0 and ***
***                           epsrel < max(50 * rel.mach.acc., 0.5d-28)),   ***
***                           or limit < 1, or c = a, or c = b.             ***
***                           In all these events res, abserr, neval,       ***
***                           rlist(1), elist(1), iord(1) and last are set  ***
***                           to zero. alist(1) and blist(1) are set to a   ***
***                           and b respectively.                           ***
***                                                                         ***
***    alist    vector of dimension at least limit, the first last          ***
***             elements of which are the left end points of the            ***
***             subintervals in the partition of the given integration      ***
***             range (a,b)                                                 *** 
***                                                                         ***
***    blist    vector of dimension at least limit, the first last          ***
***             elements of which are the right end points of the           ***
***             subintervals in the partition of the given integration      ***
***             range (a,b)                                                 ***  
***                                                                         ***
***    rlist    vector of dimension at least limit, the first last          ***
***             elements of which are the integral approximations on the    ***
***             subintervals                                                ***
***                                                                         ***
***    elist    vector of dimension at least limit, the first last          ***
***             elements of which are the moduli of the absolute error      ***
***             estimates on the subintervals                               ***
***                                                                         ***
***    last     number of subintervals actually produced in the subdivision ***
***             process                                                     ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    limit    gives an upper bound on the number of subintervals in the   ***
***             partition of (a,b) (at least 1)                             ***
***                                                                         ***
***    iord     vector of dimension at least limit, the first k elements    ***
***             of which are pointers to the error estimates over the       ***
***             subintervals, so that elist(iord(1)), ..., elist(iord(k))   ***
***             form a decreasing sequence, with k = last if last <=        ***
***             (limit/2 + 2), and k = limit + 1 - last otherwise           ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/

    int limit = LIMIT;
    double res;

    dqawce_(f, &a, &b, &c, &epsabs, &epsrel, &limit, &res, abserr, neval, ier, alist, blist, rlist, elist, iord, last);

    return res;
}
